// -*- Mode: C++; c-basic-offset: 2; indent-tabs-mode: nil; c-brace-offset: 0; -*-
/***************************************************************************
 *   Copyright (C) 2005 by Wilfried Huss                                   *
 *   Wilfried.Huss@gmx.at                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.          *
 ***************************************************************************/

#include <config.h>

#include "toolindicator.h"

#include <QPainter>


ToolIndicator::ToolIndicator(QWidget* parent)
  : QWidget(parent), currentTool(0)
{
  setMinimumSize(18, 18);
}


void ToolIndicator::slotRegisterTool(int id, QPixmap pixmap)
{
  pixmaps[id] = pixmap;
  if (currentTool == id)
    update();
}


void ToolIndicator::slotSwitchTool(int id)
{
  currentTool = id;
  update();
}


void ToolIndicator::slotClear()
{
  currentTool = 0;
  pixmaps.clear();
}


void ToolIndicator::paintEvent(QPaintEvent*)
{
  QPainter p(this);
  if (pixmaps.contains(currentTool))
  {
    QSize point = (size() - pixmaps[currentTool].size())/2;
    p.drawPixmap(point.width(), point.height(), pixmaps[currentTool]);
  }
}

#include "toolindicator.moc"
